import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:rxdart/subjects.dart';
import 'package:uuid/uuid.dart';
import 'package:video_player/video_player.dart';

import '../../../constant/value/color.dart';
import '../../../constant/value/instance_container.dart';
import '../../../constant/value/network.dart';
import '../../../delegate/route.dart';
import '../../../util/instance_register.dart';
import '../../delegate/player_controller_panel.dart';
import '../../delegate/player_switch.dart';
import 'panel/native_player_state_panel.dart';
import 'panel/native_player_texture_panel.dart';
import 'panel/player_control_panel.dart';
import 'panel/player_loading_panel.dart';
import 'panel/player_pause_image_panel.dart';

class PlayerView extends StatefulWidget {
  final String videoUrl;
  final double? aspectRatio;
  final String? pauseImagePath;
  final VoidCallback? onPauseImageTap;

  const PlayerView({
    super.key,
    required this.videoUrl,
    this.aspectRatio,
    this.pauseImagePath,
    this.onPauseImageTap,
  });

  @override
  State<PlayerView> createState() => _PlayerViewState();
}

class _PlayerViewState extends State<PlayerView>
    implements PlayerControlPanelDelegate {
  static const int _forwardBackwardSec = 15;
  final String identity = const Uuid().v4();
  late bool _isReady;
  late bool _isFullscreen;
  late Duration _duration;
  late double _volume;
  late OverlayEntry _fullscreenOverlayEntry;
  late VideoPlayerController _controller;
  late final StreamController<bool> _bufferingStateStreamController;
  late final StreamController<bool> _playingStateStreamController;
  late final StreamController<bool> _muteStateStreamController;
  late final StreamController<bool> _finishStateStreamController;
  late final StreamController<bool> _loopingStateStreamController;
  late final StreamController<double> _playbackSpeedStateStreamController;
  late final StreamController<Duration> _positionStateStreamController;
  late final StreamController<Duration> _durationStateStreamController;
  late final StreamController<List<DurationRange>>
      _bufferedStateStreamController;

  @override
  void initState() {
    super.initState();
    _fullscreenOverlayEntry = OverlayEntry(
      builder: buildFullscreen,
    );
    setState(() {
      _isReady = false;
      _isFullscreen = false;
    });
    Map<String, String> httpHeaders = {};
    if (!kIsWeb) {
      httpHeaders.addAll({
        NetworkHeaderKey.origin: NetworkHeaderValue.defaultMobileOrigin,
        NetworkHeaderKey.referer: NetworkHeaderValue.defaultMobileReferer,
      });
    }
    _controller = VideoPlayerController.network(widget.videoUrl,
        httpHeaders: httpHeaders);
    _controller.initialize().then((value) {
      Map<String, PlayerSwitchDelegate> playerSwitchDelegateMap =
          InstanceRegister.resolve<Map<String, PlayerSwitchDelegate>>(
              InstanceContainer.playerSwitchDelegateMap);
      playerSwitchDelegateMap[identity] =
          _NativePlayerSwitch(controller: _controller);
      _controller.play().then((value) {
        setState(() {
          _volume = _controller.value.volume;
          _duration = _controller.value.duration;
          _isReady = true;
        });
      });
    });
    _bufferingStateStreamController = BehaviorSubject<bool>();
    _playingStateStreamController = BehaviorSubject<bool>();
    _muteStateStreamController = BehaviorSubject<bool>();
    _finishStateStreamController = BehaviorSubject<bool>();
    _loopingStateStreamController = BehaviorSubject<bool>();
    _playbackSpeedStateStreamController = BehaviorSubject<double>();
    _positionStateStreamController = BehaviorSubject<Duration>();
    _durationStateStreamController = BehaviorSubject<Duration>();
    _bufferedStateStreamController = BehaviorSubject<List<DurationRange>>();
  }

  @override
  void dispose() {
    Map<String, PlayerSwitchDelegate> playerSwitchDelegateMap =
        InstanceRegister.resolve<Map<String, PlayerSwitchDelegate>>(
            InstanceContainer.playerSwitchDelegateMap);
    playerSwitchDelegateMap.remove(identity);
    _bufferingStateStreamController.close();
    _playingStateStreamController.close();
    _muteStateStreamController.close();
    _finishStateStreamController.close();
    _loopingStateStreamController.close();
    _playbackSpeedStateStreamController.close();
    _positionStateStreamController.close();
    _durationStateStreamController.close();
    _bufferedStateStreamController.close();
    _controller.dispose();
    super.dispose();
  }

  @override
  void setState(VoidCallback fn) {
    super.setState(fn);
    _fullscreenOverlayEntry.markNeedsBuild();
  }

  @override
  Widget build(BuildContext context) {
    if (_isFullscreen) {
      return WillPopScope(
        onWillPop: () async {
          onFireExitFullscreen();
          return false;
        },
        child: const SizedBox.shrink(),
      );
    }
    if (!_isReady) {
      return _getPreparingWidget(context);
    }
    return AspectRatio(
      aspectRatio: widget.aspectRatio ?? 16 / 9,
      child: _getPlayerWidget(),
    );
  }

  Widget _getPreparingWidget(BuildContext context) {
    return Container(
      color: CustomColors.transparent,
      child: const Center(
        child: SizedBox(
          height: 35,
          width: 35,
          child: CircularProgressIndicator(
            strokeWidth: 3,
            valueColor: AlwaysStoppedAnimation<Color>(CustomColors.lightGray),
          ),
        ),
      ),
    );
  }

  Widget buildFullscreen(BuildContext context) {
    return Positioned.fill(
      child: Scaffold(
        body: _getPlayerWidget(),
      ),
    );
  }

  Widget _getPlayerWidget() {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        return Container(
          width: constraints.maxWidth,
          height: constraints.maxWidth,
          color: CustomColors.black,
          child: Stack(
            children: [
              NativePlayerTexturePanelView(
                controller: _controller,
              ),
              NativePlayerStatePanelView(
                controller: _controller,
                bufferingStateStreamController: _bufferingStateStreamController,
                playingStateStreamController: _playingStateStreamController,
                muteStateStreamController: _muteStateStreamController,
                finishStateStreamController: _finishStateStreamController,
                loopingStateStreamController: _loopingStateStreamController,
                playbackSpeedStateStreamController:
                    _playbackSpeedStateStreamController,
                positionStateStreamController: _positionStateStreamController,
                bufferedStateStreamController: _bufferedStateStreamController,
              ),
              PlayerControlPanelView(
                delegate: this,
                fullscreen: _isFullscreen,
                forwardHint: "$_forwardBackwardSec 秒",
                backwardHint: "$_forwardBackwardSec 秒",
                duration: _duration,
                bufferingStateStreamController: _bufferingStateStreamController,
                playingStateStreamController: _playingStateStreamController,
                muteStateStreamController: _muteStateStreamController,
                finishStateStreamController: _finishStateStreamController,
                loopingStateStreamController: _loopingStateStreamController,
                playbackSpeedStateStreamController:
                    _playbackSpeedStateStreamController,
                positionStateStreamController: _positionStateStreamController,
                bufferedStateStreamController: _bufferedStateStreamController,
              ),
              PlayerLoadingPanelView(
                bufferingStateStreamController: _bufferingStateStreamController,
              ),
              if (widget.pauseImagePath != null)
                PlayerPauseImagePanelView(
                  playingStateStreamController: _playingStateStreamController,
                  finishStateStreamController: _finishStateStreamController,
                  pauseImagePath: widget.pauseImagePath!,
                  onClose: onFirePlay,
                  onTap: widget.onPauseImageTap,
                )
            ],
          ),
        );
      },
    );
  }

  @override
  void onFireBackward() {
    _controller.position.then((value) {
      if (value != null) {
        Duration target = value - const Duration(seconds: _forwardBackwardSec);
        _controller.seekTo(target.inSeconds <= 0 ? Duration.zero : target);
      }
    });
  }

  @override
  void onFireEnterFullscreen() {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    _controller.play().then((value) {
      Overlay.of(context, rootOverlay: true).insert(_fullscreenOverlayEntry);
      routeDelegate.toggleFullscreen(true).then((value) {
        setState(() {
          _isFullscreen = true;
        });
      });
    });
  }

  @override
  void onFireExitFullscreen() {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    _controller.play().then((value) {
      routeDelegate.toggleFullscreen(false).then((value) {
        _fullscreenOverlayEntry.remove();
        setState(() {
          _isFullscreen = false;
        });
      });
    });
  }

  @override
  void onFireForward() {
    _controller.position.then((value) {
      if (value != null) {
        Duration duration = _controller.value.duration;
        Duration target = value + const Duration(seconds: _forwardBackwardSec);
        _controller
            .seekTo(target.inSeconds >= duration.inSeconds ? duration : target);
      }
    });
  }

  @override
  void onFireLooping() {
    _controller.setLooping(true);
  }

  @override
  void onFireMute() {
    _controller.setVolume(0.0);
  }

  @override
  void onFirePause() {
    _controller.pause();
  }

  @override
  void onFirePlay() {
    _controller.play();
  }

  @override
  void onFireRestart() {
    _controller.seekTo(Duration.zero).then((value) {
      _controller.play();
    });
  }

  @override
  void onFireSeekTo(Duration position) {
    _controller.seekTo(position);
  }

  @override
  void onFireSetPlaybackSpeed(double speed) {
    _controller.setPlaybackSpeed(speed);
  }

  @override
  void onFireUnLooping() {
    _controller.setLooping(false);
  }

  @override
  void onFireUnMute() {
    _controller.setVolume(_volume);
  }
}

class _NativePlayerSwitch implements PlayerSwitchDelegate {
  final VideoPlayerController controller;

  _NativePlayerSwitch({required this.controller});

  @override
  void firePause() {
    controller.pause();
  }

  @override
  void firePlay() {
    controller.play();
  }
}
