import 'dart:async';

import 'package:flutter/material.dart';
import 'package:video_player/video_player.dart';

import '../../../../constant/enum/transition.dart';
import '../../../../constant/value/animation.dart';
import '../../../../constant/value/color.dart';
import '../../../../constant/value/localization.dart';
import '../../../../constant/value/size.dart';
import '../../../../util/format_parser.dart';
import '../../../../util/localization.dart';
import '../../../delegate/bezier_curve.dart';
import '../../../delegate/player_controller_panel.dart';
import '../../bezier_curve_wrapper.dart';
import '../../ripple_wrapper.dart';
import '../../transition_wrapper.dart';

class PlayerControlPanelView extends StatefulWidget {
  final PlayerControlPanelDelegate delegate;
  final bool fullscreen;
  final String forwardHint, backwardHint;
  final Duration duration;
  final StreamController<bool> bufferingStateStreamController;
  final StreamController<bool> playingStateStreamController;
  final StreamController<bool> muteStateStreamController;
  final StreamController<bool> finishStateStreamController;
  final StreamController<bool> loopingStateStreamController;
  final StreamController<double> playbackSpeedStateStreamController;
  final StreamController<Duration> positionStateStreamController;
  final StreamController<List<DurationRange>> bufferedStateStreamController;

  const PlayerControlPanelView({
    super.key,
    required this.delegate,
    required this.fullscreen,
    required this.forwardHint,
    required this.backwardHint,
    required this.duration,
    required this.bufferingStateStreamController,
    required this.playingStateStreamController,
    required this.muteStateStreamController,
    required this.finishStateStreamController,
    required this.loopingStateStreamController,
    required this.playbackSpeedStateStreamController,
    required this.positionStateStreamController,
    required this.bufferedStateStreamController,
  });

  @override
  State<PlayerControlPanelView> createState() => _PlayerControlPanelViewState();
}

class _PlayerControlPanelViewState extends State<PlayerControlPanelView> {
  late final StreamController<bool> _controlVisibleStreamController;
  late final StreamController<bool> _settingsVisibleStreamController;
  late final StreamController<bool> _speedSelectVisibleStreamController;
  bool _controlVisibleState = false;
  bool _settingsVisibleState = false;
  bool _speedSelectVisibleState = false;

  @override
  void initState() {
    super.initState();
    _controlVisibleStreamController = StreamController<bool>();
    _settingsVisibleStreamController = StreamController<bool>();
    _speedSelectVisibleStreamController = StreamController<bool>();
  }

  @override
  void dispose() {
    _controlVisibleStreamController.close();
    _settingsVisibleStreamController.close();
    _speedSelectVisibleStreamController.close();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<bool>(
        initialData: true,
        stream: widget.bufferingStateStreamController.stream,
        builder: (BuildContext bufferingStateContext,
            AsyncSnapshot<bool> bufferingStateSnapshot) {
          return _getPlayerControlPanelWidget(bufferingStateSnapshot.data!);
        });
  }

  Widget _getPlayerControlPanelWidget(bool bufferingState) {
    return StreamBuilder<bool>(
        initialData: true,
        stream: _controlVisibleStreamController.stream,
        builder: (BuildContext controlVisibleStateContext,
            AsyncSnapshot<bool> controlVisibleStateSnapshot) {
          return GestureDetector(
            onTap: _onToggleControlWidget,
            child: _getControlWidget(
                bufferingState, controlVisibleStateSnapshot.data!),
          );
        });
  }

  Widget _getControlWidget(bool bufferingState, bool controlVisible) {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        return TransitionWrapper(
          transitionKey: controlVisible && !bufferingState
              ? TransitionKey.fadeIn
              : TransitionKey.fadeOut,
          duration: AnimationDuration.short,
          child: IgnorePointer(
            ignoring: !controlVisible && bufferingState,
            child: Stack(
              children: [
                Positioned(
                  width: constraints.maxWidth * 0.5,
                  height: constraints.maxHeight,
                  left: 0.0,
                  child: _GestureBackwardWidget(
                    hint: widget.backwardHint,
                    event: widget.delegate.onFireBackward,
                  ),
                ),
                Positioned(
                  width: constraints.maxWidth * 0.5,
                  height: constraints.maxHeight,
                  right: 0.0,
                  child: _GestureForwardWidget(
                    hint: widget.forwardHint,
                    event: widget.delegate.onFireForward,
                  ),
                ),
                Positioned(
                  width: constraints.maxWidth,
                  height: 20,
                  bottom:
                      constraints.maxHeight * (widget.fullscreen ? 0.1 : 0.15),
                  child: _PlayerBuffedBarView(
                    duration: widget.duration,
                    bufferedStateStreamController:
                        widget.bufferedStateStreamController,
                  ),
                ),
                Positioned(
                  width: constraints.maxWidth,
                  height: 20,
                  bottom:
                      constraints.maxHeight * (widget.fullscreen ? 0.1 : 0.15),
                  child: _PlayerProgressBarView(
                    duration: widget.duration,
                    positionStateStreamController:
                        widget.positionStateStreamController,
                    onFireSeekTo: widget.delegate.onFireSeekTo,
                  ),
                ),
                Positioned(
                  width: constraints.maxWidth * (widget.fullscreen ? 0.2 : 0.4),
                  bottom:
                      constraints.maxHeight * (widget.fullscreen ? 0.1 : 0.15),
                  right: 0,
                  child: StreamBuilder<bool>(
                      stream: _settingsVisibleStreamController.stream,
                      builder: (BuildContext settingsVisibleStateContext,
                          AsyncSnapshot<bool> settingsVisibleStateSnapshot) {
                        if (!settingsVisibleStateSnapshot.hasData) {
                          return const SizedBox.shrink();
                        }
                        return _PlayerSettingsView(
                          visible: settingsVisibleStateSnapshot.data!,
                          loopingStateStreamController:
                              widget.loopingStateStreamController,
                          onTapSpeed: _onToggleSpeedSelectWidget,
                          onFireLooping: widget.delegate.onFireLooping,
                          onFireUnLooping: widget.delegate.onFireUnLooping,
                        );
                      }),
                ),
                Positioned(
                  width:
                      constraints.maxWidth * (widget.fullscreen ? 0.1 : 0.15),
                  bottom:
                      constraints.maxHeight * (widget.fullscreen ? 0.1 : 0.15),
                  right: 0,
                  child: StreamBuilder<bool>(
                      stream: _speedSelectVisibleStreamController.stream,
                      builder: (BuildContext speedSelectVisibleStateContext,
                          AsyncSnapshot<bool> speedSelectVisibleStateSnapshot) {
                        if (!speedSelectVisibleStateSnapshot.hasData) {
                          return const SizedBox.shrink();
                        }
                        return _PlayerSpeedSelectView(
                          visible: speedSelectVisibleStateSnapshot.data!,
                          playbackSpeedStateStreamController:
                              widget.playbackSpeedStateStreamController,
                          onFireSetPlaybackSpeed:
                              widget.delegate.onFireSetPlaybackSpeed,
                        );
                      }),
                ),
                Positioned(
                  width: constraints.maxWidth,
                  height:
                      constraints.maxHeight * (widget.fullscreen ? 0.1 : 0.15),
                  bottom: 0,
                  child: Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 6),
                    child: Row(
                      children: [
                        StreamBuilder<bool>(
                            initialData: true,
                            stream: widget.playingStateStreamController.stream,
                            builder: (BuildContext playingStateContext,
                                AsyncSnapshot<bool> playingStateSnapshot) {
                              return _PlayerIconWidget(
                                icon: playingStateSnapshot.data!
                                    ? Icons.pause_rounded
                                    : Icons.play_arrow_rounded,
                                onTap: playingStateSnapshot.data!
                                    ? widget.delegate.onFirePause
                                    : widget.delegate.onFirePlay,
                              );
                            }),
                        StreamBuilder<bool>(
                            initialData: false,
                            stream: widget.muteStateStreamController.stream,
                            builder: (BuildContext muteStateContext,
                                AsyncSnapshot<bool> muteStateSnapshot) {
                              return _PlayerIconWidget(
                                icon: muteStateSnapshot.data!
                                    ? Icons.volume_up_rounded
                                    : Icons.volume_off_rounded,
                                onTap: muteStateSnapshot.data!
                                    ? widget.delegate.onFireUnMute
                                    : widget.delegate.onFireMute,
                              );
                            }),
                        Expanded(
                            child: _PlayerDurationWidget(
                          duration: widget.duration,
                          positionStateStreamController:
                              widget.positionStateStreamController,
                        )),
                        _PlayerIconWidget(
                          icon: Icons.settings,
                          onTap: _onToggleSettingsWidget,
                        ),
                        _PlayerIconWidget(
                          icon: Icons.fullscreen,
                          onTap: widget.fullscreen
                              ? widget.delegate.onFireExitFullscreen
                              : widget.delegate.onFireEnterFullscreen,
                        ),
                      ],
                    ),
                  ),
                ),
                Positioned(
                  width: 60,
                  height: 60,
                  top: constraints.maxHeight / 2 - 30,
                  left: constraints.maxWidth / 2 - 30,
                  child: _PlayerCenterButtonView(
                    playingStateStreamController:
                        widget.playingStateStreamController,
                    finishStateStreamController:
                        widget.finishStateStreamController,
                    onFirePlay: widget.delegate.onFirePlay,
                    onFirePause: widget.delegate.onFirePause,
                    onFireRestart: widget.delegate.onFireRestart,
                  ),
                ),
              ],
            ),
          ),
        );
      },
    );
  }

  void _onToggleControlWidget() {
    _controlVisibleState = !_controlVisibleState;
    _controlVisibleStreamController.add(_controlVisibleState);
  }

  void _onToggleSettingsWidget() {
    if (_speedSelectVisibleState) {
      _speedSelectVisibleState = false;
      _speedSelectVisibleStreamController.add(_speedSelectVisibleState);
    } else {
      _settingsVisibleState = !_settingsVisibleState;
      _settingsVisibleStreamController.add(_settingsVisibleState);
    }
  }

  void _onToggleSpeedSelectWidget() {
    _speedSelectVisibleState = !_speedSelectVisibleState;
    _speedSelectVisibleStreamController.add(_speedSelectVisibleState);
    if (_settingsVisibleState) {
      _settingsVisibleState = !_settingsVisibleState;
      _settingsVisibleStreamController.add(_settingsVisibleState);
    }
  }
}

class _PlayerIconWidget extends StatelessWidget {
  final IconData icon;
  final VoidCallback? onTap;

  const _PlayerIconWidget({required this.icon, this.onTap});

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      return Padding(
        padding: const EdgeInsets.symmetric(horizontal: 2),
        child: MouseRegion(
          cursor: SystemMouseCursors.click,
          child: GestureDetector(
            onTap: onTap,
            child: Icon(
                size: constraints.maxHeight * 0.7,
                icon,
                color: CustomColors.white),
          ),
        ),
      );
    });
  }
}

class _PlayerDurationWidget extends StatelessWidget {
  final Duration duration;
  final StreamController<Duration> positionStateStreamController;

  const _PlayerDurationWidget({
    required this.duration,
    required this.positionStateStreamController,
  });

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<Duration>(
        initialData: Duration.zero,
        stream: positionStateStreamController.stream,
        builder: (BuildContext positionStateContext,
            AsyncSnapshot<Duration> positionStateSnapshot) {
          return _getPlayerDurationWidget(positionStateSnapshot.data!);
        });
  }

  Widget _getPlayerDurationWidget(Duration position) {
    String positionText = FormatParser.parseVideoDuration(position);
    String durationText = FormatParser.parseVideoDuration(duration);
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 6),
      child: Text(
        "$positionText / $durationText",
        style: const TextStyle(
          fontSize: FontSize.medium,
          color: CustomColors.white,
        ),
      ),
    );
  }
}

class _PlayerSettingsView extends StatelessWidget {
  final bool visible;
  final StreamController<bool> loopingStateStreamController;
  final VoidCallback? onTapSpeed;
  final VoidCallback? onFireLooping;
  final VoidCallback? onFireUnLooping;

  const _PlayerSettingsView({
    required this.visible,
    required this.loopingStateStreamController,
    required this.onTapSpeed,
    this.onFireLooping,
    this.onFireUnLooping,
  });

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<bool>(
        initialData: false,
        stream: loopingStateStreamController.stream,
        builder: (BuildContext loopingStateContext,
            AsyncSnapshot<bool> loopingStateSnapshot) {
          return _getSettingsDetailWidget(
              loopingStateContext, loopingStateSnapshot.data!);
        });
  }

  Widget _getSettingsDetailWidget(BuildContext context, bool loopingState) {
    return TransitionWrapper(
      transitionKey: visible ? TransitionKey.fadeIn : TransitionKey.fadeOut,
      duration: AnimationDuration.short,
      child: IgnorePointer(
        ignoring: !visible,
        child: Card(
          color: CustomColors.transparent,
          child: Column(
            children: [
              RippleWrapper(
                splashColor: CustomColors.gray.withOpacity(0.4),
                onTap: _onTapSpeed,
                child: _getSettingsDetailItemWidget(
                  insidePaddingVertical: 8,
                  tag: Localization.getString(
                      context, LocalizationStrings.speed),
                  actionWidget: const Icon(
                    Icons.chevron_right,
                    color: CustomColors.gray,
                    size: IconSize.medium,
                  ),
                ),
              ),
              RippleWrapper(
                splashColor: CustomColors.gray.withOpacity(0.4),
                child: _getSettingsDetailItemWidget(
                  insidePaddingVertical: 0,
                  tag: Localization.getString(
                      context, LocalizationStrings.looping),
                  onTap: () {},
                  actionWidget: Transform.scale(
                    scale: 0.7,
                    alignment: Alignment.centerRight,
                    child: Switch(
                      materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
                      trackColor: MaterialStateProperty.resolveWith<Color?>(
                        (Set<MaterialState> states) {
                          if (states.contains(MaterialState.selected)) {
                            return CustomColors.smokeGray;
                          }
                          return CustomColors.white;
                        },
                      ),
                      thumbColor: MaterialStateProperty.resolveWith<Color?>(
                        (Set<MaterialState> states) {
                          if (states.contains(MaterialState.selected)) {
                            return CustomColors.white;
                          }
                          return CustomColors.ravenBlack;
                        },
                      ),
                      value: loopingState,
                      onChanged: _onChangedLooping,
                    ),
                  ),
                ),
              )
            ],
          ),
        ),
      ),
    );
  }

  Widget _getSettingsDetailItemWidget(
      {required String tag,
      required Widget actionWidget,
      VoidCallback? onTap,
      double? insidePaddingVertical}) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 1),
      child: RippleWrapper(
        borderRadius: BorderRadius.circular(6),
        splashColor: CustomColors.slateGray,
        backgroundColor: CustomColors.graphiteGray,
        onTap: onTap,
        child: Padding(
          padding: EdgeInsets.symmetric(
              vertical: insidePaddingVertical ?? 0, horizontal: 5),
          child: Row(
            children: [
              Expanded(
                child: Padding(
                  padding: const EdgeInsets.only(left: 10),
                  child: Text(
                    tag,
                    style: const TextStyle(
                      fontSize: FontSize.medium,
                      color: CustomColors.white,
                    ),
                  ),
                ),
              ),
              actionWidget
            ],
          ),
        ),
      ),
    );
  }

  void _onTapSpeed() {
    onTapSpeed?.call();
  }

  void _onChangedLooping(bool value) {
    if (value) {
      onFireLooping?.call();
    } else {
      onFireUnLooping?.call();
    }
  }
}

class _PlayerSpeedSelectView extends StatelessWidget {
  static const List<double> _speedList = [0.5, 0.75, 1, 1.25, 1.5, 2];
  static const double defaultSpeed = 1.0;
  final bool visible;
  final StreamController<double> playbackSpeedStateStreamController;
  final void Function(double speed)? onFireSetPlaybackSpeed;

  const _PlayerSpeedSelectView({
    required this.visible,
    required this.playbackSpeedStateStreamController,
    required this.onFireSetPlaybackSpeed,
  });

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<double>(
        initialData: defaultSpeed,
        stream: playbackSpeedStateStreamController.stream,
        builder: (BuildContext playbackSpeedStateContext,
            AsyncSnapshot<double> playbackSpeedStateSnapshot) {
          return _getSpeedSelectWidget(
              playbackSpeedStateContext, playbackSpeedStateSnapshot.data!);
        });
  }

  Widget _getSpeedSelectWidget(BuildContext context, double speed) {
    return TransitionWrapper(
      transitionKey: visible ? TransitionKey.fadeIn : TransitionKey.fadeOut,
      duration: AnimationDuration.short,
      child: IgnorePointer(
        ignoring: !visible,
        child: Card(
          color: CustomColors.transparent,
          child: Column(
            children: List.generate(_speedList.length, (index) {
              return RippleWrapper(
                child: _getSpeedSelectItemWidget(
                  color: _speedList[index] == speed ? CustomColors.green : null,
                  speedTag: _speedList[index] == defaultSpeed
                      ? Localization.getString(
                          context, LocalizationStrings.normal)
                      : _speedList[index].toString(),
                  onTap: () => onFireSetPlaybackSpeed?.call(_speedList[index]),
                ),
              );
            }).toList(),
          ),
        ),
      ),
    );
  }

  Widget _getSpeedSelectItemWidget(
      {required String speedTag, Color? color, VoidCallback? onTap}) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 1),
      child: RippleWrapper(
        borderRadius: BorderRadius.circular(6),
        splashColor: CustomColors.slateGray,
        backgroundColor: CustomColors.graphiteGray,
        onTap: onTap,
        child: Padding(
          padding: const EdgeInsets.all(4),
          child: Row(
            children: [
              Expanded(
                child: Center(
                  child: Text(
                    speedTag,
                    style: TextStyle(
                      fontSize: FontSize.medium,
                      color: color ?? CustomColors.white,
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}

class _PlayerProgressBarView extends StatefulWidget {
  final Duration duration;
  final StreamController<Duration> positionStateStreamController;
  final void Function(Duration position)? onFireSeekTo;

  const _PlayerProgressBarView({
    required this.duration,
    required this.positionStateStreamController,
    this.onFireSeekTo,
  });

  @override
  State<_PlayerProgressBarView> createState() => _PlayerProgressBarViewState();
}

class _PlayerProgressBarViewState extends State<_PlayerProgressBarView> {
  late double _progressValue;
  late bool _onProgressDragging;

  @override
  void initState() {
    super.initState();
    setState(() {
      _progressValue = 0;
      _onProgressDragging = false;
    });
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<Duration>(
        initialData: Duration.zero,
        stream: widget.positionStateStreamController.stream,
        builder: (BuildContext positionStateContext,
            AsyncSnapshot<Duration> positionStateSnapshot) {
          return _getPlayerProgressBarWidget(positionStateSnapshot.data!);
        });
  }

  Widget _getPlayerProgressBarWidget(Duration position) {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 10),
      child: LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          return SliderTheme(
            data: SliderThemeData(
              thumbColor: CustomColors.green.withOpacity(0.8),
              activeTrackColor: CustomColors.green.withOpacity(0.8),
              inactiveTrackColor: CustomColors.gray.withOpacity(0.45),
              overlayShape: SliderComponentShape.noOverlay,
              trackHeight: constraints.maxHeight * 0.10,
              trackShape: const RoundedRectSliderTrackShape(),
              thumbShape: RoundSliderThumbShape(
                elevation: 0,
                enabledThumbRadius: constraints.maxHeight * 0.25,
                pressedElevation: 0,
              ),
            ),
            child: Slider(
              value: _onProgressDragging
                  ? _progressValue
                  : 100 * (position.inSeconds / widget.duration.inSeconds),
              min: 0,
              max: 100,
              onChangeStart: _onChangeStart,
              onChanged: _onChanged,
              onChangeEnd: _onChangeEnd,
            ),
          );
        },
      ),
    );
  }

  void _onChangeStart(double value) {
    setState(() {
      _onProgressDragging = true;
      _progressValue = value;
    });
  }

  void _onChanged(double value) {
    setState(() {
      _progressValue = value;
    });
  }

  void _onChangeEnd(double value) {
    int seconds = (widget.duration.inSeconds * value / 100).round().toInt();
    if (seconds <= 0) {
      widget.onFireSeekTo?.call(Duration.zero);
    } else if (seconds >= widget.duration.inSeconds) {
      widget.onFireSeekTo?.call(widget.duration);
    } else {
      widget.onFireSeekTo?.call(Duration(seconds: seconds));
    }
    setState(() {
      _onProgressDragging = false;
    });
  }
}

class _PlayerBuffedBarView extends StatelessWidget {
  final Duration duration;
  final StreamController<List<DurationRange>> bufferedStateStreamController;

  const _PlayerBuffedBarView({
    required this.duration,
    required this.bufferedStateStreamController,
  });

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<List<DurationRange>>(
        initialData: const [],
        stream: bufferedStateStreamController.stream,
        builder: (BuildContext bufferedStateContext,
            AsyncSnapshot<List<DurationRange>> bufferedStateSnapshot) {
          return _getBuffedBarWidget(bufferedStateSnapshot.data!);
        });
  }

  Widget _getBuffedBarWidget(List<DurationRange> buffered) {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 10),
      child: LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          return Padding(
            padding:
                EdgeInsets.symmetric(horizontal: constraints.maxHeight * 0.2),
            child: Stack(
              children: [
                for (var buffer in buffered)
                  Positioned(
                    left: constraints.maxWidth *
                        _calculateValue(
                            buffer.start.inSeconds, duration.inSeconds),
                    bottom: constraints.maxHeight * 0.45,
                    child: Container(
                      width: constraints.maxWidth *
                          _calculateValue(
                              buffer.end.inSeconds - buffer.start.inSeconds,
                              duration.inSeconds),
                      height: constraints.maxHeight * 0.10,
                      decoration: BoxDecoration(
                        color: Colors.grey,
                        borderRadius:
                            BorderRadius.circular(constraints.maxHeight * 0.25),
                      ),
                    ),
                  ),
              ],
            ),
          );
        },
      ),
    );
  }

  double _calculateValue(int positionSeconds, int durationSeconds) {
    if (durationSeconds == 0) {
      return 0;
    }
    return positionSeconds / durationSeconds;
  }
}

class _PlayerCenterButtonView extends StatelessWidget {
  final StreamController<bool> playingStateStreamController;
  final StreamController<bool> finishStateStreamController;
  final VoidCallback? onFirePlay;
  final VoidCallback? onFirePause;
  final VoidCallback? onFireRestart;

  const _PlayerCenterButtonView({
    required this.playingStateStreamController,
    required this.finishStateStreamController,
    this.onFirePlay,
    this.onFirePause,
    this.onFireRestart,
  });

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<bool>(
        initialData: true,
        stream: playingStateStreamController.stream,
        builder: (BuildContext playingStateContext,
            AsyncSnapshot<bool> playingStateSnapshot) {
          return StreamBuilder<bool>(
              initialData: false,
              stream: finishStateStreamController.stream,
              builder: (BuildContext finishStateContext,
                  AsyncSnapshot<bool> finishStateSnapshot) {
                return _getCenterButtonWidget(
                    playingStateSnapshot.data!, finishStateSnapshot.data!);
              });
        });
  }

  Widget _getCenterButtonWidget(bool playingState, bool finishState) {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        return Card(
          color: CustomColors.transparent,
          elevation: 0,
          child: RippleWrapper(
            borderRadius: BorderRadius.circular(constraints.maxHeight / 2),
            splashColor: CustomColors.gray.withOpacity(0.4),
            onTap: playingState
                ? onFirePause
                : finishState
                    ? onFireRestart
                    : onFirePlay,
            decoration: BoxDecoration(
              shape: BoxShape.circle,
              color: CustomColors.midnightBlack.withOpacity(0.4),
              boxShadow: null,
            ),
            child: Icon(
              finishState
                  ? Icons.replay_rounded
                  : playingState
                      ? Icons.pause_rounded
                      : Icons.play_arrow_rounded,
              color: CustomColors.white,
              size: constraints.maxHeight * 0.5,
            ),
          ),
        );
      },
    );
  }
}

class _GestureBackwardWidget extends StatefulWidget {
  final String hint;
  final VoidCallback? event;

  const _GestureBackwardWidget({required this.hint, this.event});

  @override
  State<_GestureBackwardWidget> createState() => _GestureBackwardWidgetState();
}

class _GestureBackwardWidgetState extends State<_GestureBackwardWidget>
    implements BezierCurveDelegate {
  late bool _showHolder;

  @override
  void initState() {
    super.initState();
    _showHolder = false;
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onDoubleTap: _onDoubleTap,
      child: SizedBox(
        width: double.infinity,
        height: double.infinity,
        child: TransitionWrapper(
          transitionKey:
              _showHolder ? TransitionKey.fadeIn : TransitionKey.fadeOut,
          duration: AnimationDuration.short,
          child: BezierCurveWrapper(
            delegate: this,
            child: Container(
              color: Colors.grey.withOpacity(0.3),
              child: Center(
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    const Icon(
                        size: IconSize.medium,
                        Icons.fast_rewind_rounded,
                        color: CustomColors.white),
                    Text(
                      widget.hint,
                      style: const TextStyle(
                        fontSize: FontSize.medium,
                        color: CustomColors.white,
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  void _onDoubleTap() {
    setState(() {
      _showHolder = true;
    });
    widget.event?.call();
    Future.delayed(AnimationDuration.long, () {
      setState(() {
        _showHolder = false;
      });
    });
  }

  @override
  Path customClipPath(Size size) {
    final double width = size.width;
    final double height = size.height;
    final Path path = Path()
      ..lineTo(0, height)
      ..lineTo(width * 0.75, height)
      ..quadraticBezierTo(
        width,
        height * 0.5,
        width * 0.75,
        0,
      )
      ..lineTo(width * 0.75, 0)
      ..close();
    return path;
  }
}

class _GestureForwardWidget extends StatefulWidget {
  final String hint;
  final VoidCallback? event;

  const _GestureForwardWidget({required this.hint, this.event});

  @override
  State<_GestureForwardWidget> createState() => _GestureForwardWidgetState();
}

class _GestureForwardWidgetState extends State<_GestureForwardWidget>
    implements BezierCurveDelegate {
  late bool _showHolder;

  @override
  void initState() {
    super.initState();
    _showHolder = false;
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onDoubleTap: _onDoubleTap,
      child: SizedBox(
        width: double.infinity,
        height: double.infinity,
        child: TransitionWrapper(
          transitionKey:
              _showHolder ? TransitionKey.fadeIn : TransitionKey.fadeOut,
          duration: AnimationDuration.short,
          child: BezierCurveWrapper(
            delegate: this,
            child: Container(
              color: Colors.grey.withOpacity(0.3),
              child: Center(
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    const Icon(
                        size: IconSize.medium,
                        Icons.fast_forward_rounded,
                        color: CustomColors.white),
                    Text(
                      widget.hint,
                      style: const TextStyle(
                        fontSize: FontSize.medium,
                        color: CustomColors.white,
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  void _onDoubleTap() {
    setState(() {
      _showHolder = true;
    });
    widget.event?.call();
    Future.delayed(AnimationDuration.long, () {
      setState(() {
        _showHolder = false;
      });
    });
  }

  @override
  Path customClipPath(Size size) {
    final double width = size.width;
    final double height = size.height;
    final Path path = Path()
      ..moveTo(width, 0)
      ..lineTo(width, height)
      ..lineTo(width * 0.25, height)
      ..quadraticBezierTo(
        0,
        height * 0.5,
        width * 0.25,
        0,
      )
      ..lineTo(width, 0)
      ..close();
    return path;
  }
}
