import 'dart:async';

import 'package:flutter/material.dart';
import 'package:redinfo_integrated_ott_platform_flutter_sdk/delegate/communication_input_delegate.dart';
import 'package:redinfo_integrated_ott_platform_flutter_sdk/delegate/communication_output_delegate.dart';
import '../../constant/value/instance_container.dart';
import '../../delegate/data_service.dart';
import 'component/delegate/player_switch.dart';
import 'constant/value/assets.dart';
import 'constant/value/localization.dart';
import 'delegate/load_balancer.dart';
import 'delegate/route.dart';
import 'manager/data_service.dart';
import 'manager/load_balancer/load_balancer.dart' as load_balancer;
import 'manager/route.dart';
import 'util/assets.dart';
import 'util/instance_register.dart';
import 'util/localization.dart';

class RedInfoPornVideoApplication implements CommunicationInputDelegate {
  final CommunicationOutputDelegate communicationOutputDelegate;

  RedInfoPornVideoApplication({required this.communicationOutputDelegate});

  static Future<Map<String, dynamic>> getLocalizationMap() {
    return Localization.mapFromJsonFile(
        assets: Assets.getAssetsPath(
            "${AssetsPath.packagePrefix}/${AssetsPath.i18ZhCn}"),
        prefix: LocalizationStrings.appIdentity);
  }

  @override
  Widget getRootWidget() {
    return _Application(
      communicationOutputDelegate: communicationOutputDelegate,
    );
  }

  @override
  Future<void> popRoute() {
    return Future.sync(() {
      RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
          InstanceContainer.routeDelegate);
      return routeDelegate.pop();
    });
  }

  @override
  Future<void> fireSearch() {
    return Future.value();
  }

  @override
  Future<void> leavePage() {
    return Future.sync(() {
      Map<String, PlayerSwitchDelegate> playerSwitchDelegateMap =
          InstanceRegister.resolve<Map<String, PlayerSwitchDelegate>>(
              InstanceContainer.playerSwitchDelegateMap);
      for (var entry in playerSwitchDelegateMap.entries) {
        entry.value.firePause();
      }
      return;
    });
  }
}

class _Application extends StatefulWidget {
  final CommunicationOutputDelegate communicationOutputDelegate;

  const _Application({required this.communicationOutputDelegate});

  @override
  State<_Application> createState() => _ApplicationState();
}

class _ApplicationState extends State<_Application> {
  late final DataServiceManager _dataServiceManager;
  late final RouteManager _routeManager;
  late final load_balancer.LoadBalancerManager _loadBalancerManager;
  bool _isReady = false;

  @override
  void initState() {
    super.initState();
    //generate manager
    _dataServiceManager = DataServiceManager();
    _routeManager = RouteManager();
    _loadBalancerManager = load_balancer.LoadBalancerManager();
    //register localization prefix
    Localization.setUpPrefix(LocalizationStrings.appIdentity);
    //register assets packageName
    Assets.setUpPrefix(AssetsPath.packagePrefix);
    // clear Instance
    InstanceRegister.clear();
    // register manager as delegate
    InstanceRegister.registerSingleton<DataServiceDelegate>(
        (container) => _dataServiceManager,
        name: InstanceContainer.dataServiceDelegate);
    InstanceRegister.registerSingleton<RouteDelegate>(
        (container) => _routeManager,
        name: InstanceContainer.routeDelegate);
    InstanceRegister.registerSingleton<LoadBalancerDelegate>(
        (container) => _loadBalancerManager,
        name: InstanceContainer.loadBalancerDelegate);
    InstanceRegister.registerSingleton<CommunicationOutputDelegate>(
        (container) => widget.communicationOutputDelegate,
        name: InstanceContainer.communicationOutputDelegate);
    InstanceRegister.registerSingleton<Map<String, PlayerSwitchDelegate>>(
        (container) => {},
        name: InstanceContainer.playerSwitchDelegateMap);
    _dataServiceManager.initial().then((value) {
      setState(() {
        _isReady = true;
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    if (!_isReady) {
      return const SizedBox.shrink();
    }
    return _routeManager.getRootNavigator();
  }
}
