import 'dart:async';
import 'dart:convert';
import 'package:redinfo_integrated_ott_platform_flutter_sdk/delegate/communication_output_delegate.dart';
import '../../structure/entity/advertisement_image.dart';
import '../constant/value/instance_container.dart';
import '../constant/value/network.dart';
import '../delegate/data_service.dart';
import '../delegate/load_balancer.dart';
import '../network/client.dart';
import '../structure/entity/actor.dart';
import '../structure/entity/classification_detail.dart';
import '../structure/entity/classification_video.dart';
import '../structure/entity/director.dart';
import '../structure/entity/relative_video_collection.dart';
import '../structure/entity/settings.dart';
import '../structure/entity/video.dart';
import '../structure/entity/video_detail.dart';
import '../structure/entity/video_group.dart';
import '../structure/response/actor.dart';
import '../structure/response/advertisement_image.dart';
import '../structure/response/advertisement_image_collection.dart';
import '../structure/response/classification_detail.dart';
import '../structure/response/classification_video.dart';
import '../structure/response/director.dart';
import '../structure/response/encoding_thumbnail.dart';
import '../structure/response/relative_video_collection.dart';
import '../structure/response/settings.dart';
import '../structure/response/video.dart';
import '../structure/response/video_detail.dart';
import '../structure/response/video_group.dart';
import '../util/instance_register.dart';

class DataServiceManager implements DataServiceDelegate {
  final String apiHost = NetworkHost.defaultApiHost;
  String _apiCdnHost = NetworkHost.defaultApiCdnHost;
  String _adApiCdnHost = NetworkHost.defaultAdApiCdnHost;
  String _imageCdnHost = NetworkHost.defaultImageCdnHost;
  String _videoCdnHost = NetworkHost.defaultVideoCdnHost;

  NetworkHttpClient _getHttpClient({Map<String, String>? customHeaders}) {
    return NetworkHttpClient(customHeaders: customHeaders ?? {});
  }

  Future<T> _fireIsolate<T>(Future<T> Function(void) function) {
    LoadBalancerDelegate loadBalancerDelegate =
        InstanceRegister.resolve<LoadBalancerDelegate>(
            InstanceContainer.loadBalancerDelegate);
    return loadBalancerDelegate.runIsolate<T, void>(function, null);
  }

  Future<void> initial() {
    return _fireIsolate<Settings>((void _) {
      Completer<Settings> completer = Completer();
      NetworkHttpClient httpClient = _getHttpClient();
      String url = httpClient.buildUrl(apiHost, [NetworkPath.getSettings]);
      Map<String, String> params = <String, String>{};
      httpClient.get(url, params).then((body) {
        try {
          SettingsResponse response =
              SettingsResponse.fromJson(json.decode(body));
          completer.complete(Settings(
            apiCdnHost: response.apiCdnHost,
            adApiCdnHost: response.adApiCdnHost,
            imageCdnHost: response.imageCdnHost,
            videoCdnHost: response.videoCdnHost,
          ));
        } catch (err) {
          completer.completeError(err);
        }
      });
      return completer.future;
    }).then((settings) {
      _apiCdnHost = settings.apiCdnHost;
      _adApiCdnHost = settings.adApiCdnHost;
      _imageCdnHost = settings.imageCdnHost;
      _videoCdnHost = settings.videoCdnHost;
      return;
    });
  }

  @override
  Future<String> getBase64Cache(String key) async {
    CommunicationOutputDelegate communicationOutputDelegate =
        InstanceRegister.resolve<CommunicationOutputDelegate>(
            InstanceContainer.communicationOutputDelegate);
    return communicationOutputDelegate.getBase64ImageCache(key);
  }

  @override
  Future<void> saveBase64Cache(String key, String value) {
    CommunicationOutputDelegate communicationOutputDelegate =
        InstanceRegister.resolve<CommunicationOutputDelegate>(
            InstanceContainer.communicationOutputDelegate);
    return communicationOutputDelegate.saveBase64ImageCache(key, value);
  }

  @override
  Future<String> getBinaryImageFile(String path) {
    return _fireIsolate<String>((void _) {
      NetworkHttpClient httpClient = _getHttpClient();
      String url = httpClient.buildUrl(_imageCdnHost, [path]);
      return httpClient.get(url, {});
    });
  }

  @override
  Future<Map<String, List<AdvertisementImage>>> getAdvertisementImageListMap(
      List<String> imageRatioList) {
    return _fireIsolate<Map<String, List<AdvertisementImage>>>(
      (void _) {
        Completer<Map<String, List<AdvertisementImage>>> completer =
            Completer();
        NetworkHttpClient httpClient = _getHttpClient();
        String url = httpClient.buildUrl(
            _adApiCdnHost, [NetworkPath.getAdvertisementImageCollectionList]);
        Map<String, String> params = <String, String>{};
        params[NetworkQueryKey.platform] = NetworkParam.platformPornVideo;
        params[NetworkQueryKey.ratio] = imageRatioList.join(NetworkChar.comma);
        httpClient.get(url, params).then(
          (body) {
            try {
              Map<String, List<AdvertisementImage>> advertisementImageListMap =
                  {};
              List<AdvertisementImageCollectionResponse>
                  advertisementImageCollectionResponseList =
                  (json.decode(body) as List)
                      .map((data) =>
                          AdvertisementImageCollectionResponse.fromJson(data))
                      .toList();
              for (AdvertisementImageCollectionResponse advertisementImageCollectionResponse
                  in advertisementImageCollectionResponseList) {
                advertisementImageListMap[
                        advertisementImageCollectionResponse.ratio] =
                    List.generate(
                        advertisementImageCollectionResponse.list.length,
                        (index) {
                  AdvertisementImageResponse advertisementImageResponse =
                      advertisementImageCollectionResponse.list[index];
                  return AdvertisementImage(
                    title: advertisementImageResponse.title,
                    link: advertisementImageResponse.link,
                    description: advertisementImageResponse.description,
                    imagePath: advertisementImageResponse.thumbnailPath,
                  );
                });
              }
              completer.complete(advertisementImageListMap);
            } catch (err) {
              completer.completeError(err);
            }
          },
        );
        return completer.future;
      },
    );
  }

  @override
  Future<ClassificationDetail> getClassificationDetail(
      int videoSize, String imageRatio) {
    return _fireIsolate<ClassificationDetail>(
      (void _) {
        Completer<ClassificationDetail> completer = Completer();
        NetworkHttpClient httpClient = _getHttpClient();
        String url = httpClient
            .buildUrl(_apiCdnHost, [NetworkPath.getClassificationDetail]);
        Map<String, String> params = <String, String>{};
        params[NetworkQueryKey.videoSize] = "$videoSize";
        httpClient.get(url, params).then(
          (body) {
            try {
              ClassificationDetailResponse classificationDetailResponse =
                  ClassificationDetailResponse.fromJson(json.decode(body));
              completer.complete(
                ClassificationDetail(
                  videoGroupList: List.generate(
                      classificationDetailResponse.videoGroupList.length,
                      (index) {
                    VideoGroupResponse videoGroupResponse =
                        classificationDetailResponse.videoGroupList[index];
                    return VideoGroup(
                        id: videoGroupResponse.id,
                        title: videoGroupResponse.name,
                        collectionCount: videoGroupResponse.count);
                  }),
                  classificationVideoList: List.generate(
                      classificationDetailResponse
                          .classificationVideoList.length, (index) {
                    ClassificationVideoResponse classificationVideoResponse =
                        classificationDetailResponse
                            .classificationVideoList[index];
                    EncodingThumbnailResponse encodingThumbnailResponse =
                        classificationVideoResponse.encodingThumbnailList
                            .firstWhere(
                                (element) => element.ratio == imageRatio,
                                orElse: () => EncodingThumbnailResponse(
                                    ratio: "", dataUrl: ""));
                    return ClassificationVideo(
                        id: classificationVideoResponse.id,
                        title: classificationVideoResponse.title,
                        thumbnailPath: encodingThumbnailResponse.dataUrl);
                  }),
                ),
              );
            } catch (err) {
              completer.completeError(err);
            }
          },
        );
        return completer.future;
      },
    );
  }

  @override
  Future<VideoDetail> getVideoDetail(int videoId, String imageRatio) {
    return _fireIsolate<VideoDetail>((void _) {
      Completer<VideoDetail> completer = Completer();
      NetworkHttpClient httpClient = _getHttpClient();
      String url =
          httpClient.buildUrl(_apiCdnHost, [NetworkPath.getVideoDetail]);
      Map<String, String> params = <String, String>{};
      params[NetworkQueryKey.videoId] = "$videoId";
      httpClient.get(url, params).then((body) {
        try {
          VideoDetailResponse videoDetailResponse =
              VideoDetailResponse.fromJson(json.decode(body));
          EncodingThumbnailResponse encodingThumbnailResponse =
              videoDetailResponse.encodingThumbnailList
                  .firstWhere((element) => element.ratio == imageRatio);

          completer.complete(VideoDetail(
            id: videoDetailResponse.id,
            title: videoDetailResponse.title,
            description: videoDetailResponse.description,
            createTime: videoDetailResponse.createdTime,
            thumbnailPath: encodingThumbnailResponse.dataUrl,
            sourceUrl:
                "$_videoCdnHost${videoDetailResponse.videoMediaSourcePath}",
            videoGroupList: List.generate(
                videoDetailResponse.videoGroupList.length, (videoGroupIndex) {
              VideoGroupResponse videoGroupResponse =
                  videoDetailResponse.videoGroupList[videoGroupIndex];
              return VideoGroup(
                  id: videoGroupResponse.id,
                  title: videoGroupResponse.name,
                  collectionCount: 0);
            }),
            actorList: List.generate(videoDetailResponse.actorList.length,
                (actorIndex) {
              ActorResponse actorResponse =
                  videoDetailResponse.actorList[actorIndex];
              return Actor(id: actorResponse.id, name: actorResponse.name);
            }),
            directorList: List.generate(videoDetailResponse.directorList.length,
                (directorIndex) {
              DirectorResponse directorResponse =
                  videoDetailResponse.directorList[directorIndex];
              return Director(
                  id: directorResponse.id, name: directorResponse.name);
            }),
          ));
        } catch (err) {
          completer.completeError(err);
        }
      });
      return completer.future;
    });
  }

  @override
  Future<List<VideoGroup>> getAllVideoGroupList() {
    return _fireIsolate<List<VideoGroup>>(
      (void _) {
        Completer<List<VideoGroup>> completer = Completer();
        NetworkHttpClient httpClient = _getHttpClient();
        String url =
            httpClient.buildUrl(_apiCdnHost, [NetworkPath.getVideoGroupList]);
        Map<String, String> params = <String, String>{};
        params[NetworkQueryKey.groupSize] = NetworkParam.groupAll;
        httpClient.get(url, params).then(
          (body) {
            try {
              List<VideoGroupResponse> videoGroupResponseList =
                  (json.decode(body) as List)
                      .map((data) => VideoGroupResponse.fromJson(data))
                      .toList();
              completer.complete(
                List.generate(
                  videoGroupResponseList.length,
                  (index) {
                    VideoGroupResponse videoGroupResponse =
                        videoGroupResponseList[index];
                    return VideoGroup(
                      id: videoGroupResponse.id,
                      title: videoGroupResponse.name,
                      collectionCount: videoGroupResponse.count,
                    );
                  },
                ),
              );
            } catch (err) {
              completer.completeError(err);
            }
          },
        );
        return completer.future;
      },
    );
  }

  @override
  Future<List<Video>> getAllVideoList(
      int lower, int upper, String sort, String imageRatio) {
    return _fireIsolate<List<Video>>(
      (void _) {
        Completer<List<Video>> completer = Completer();
        NetworkHttpClient httpClient = _getHttpClient();
        String url =
            httpClient.buildUrl(_apiCdnHost, [NetworkPath.getGroupVideoList]);
        Map<String, String> params = <String, String>{};
        params[NetworkQueryKey.groupId] = NetworkParam.groupAll;
        params[NetworkQueryKey.lower] = "$lower";
        params[NetworkQueryKey.upper] = "$upper";
        params[NetworkQueryKey.sort] = sort;
        httpClient.get(url, params).then(
          (body) {
            try {
              List<VideoResponse> videoResponseList =
                  (json.decode(body) as List)
                      .map((data) => VideoResponse.fromJson(data))
                      .toList();
              completer.complete(
                List.generate(
                  videoResponseList.length,
                  (index) {
                    VideoResponse videoResponse = videoResponseList[index];
                    EncodingThumbnailResponse encodingThumbnailResponse =
                        videoResponse.encodingThumbnailList.firstWhere(
                            (element) => element.ratio == imageRatio);
                    return Video(
                      id: videoResponse.id,
                      title: videoResponse.title,
                      thumbnailPath: encodingThumbnailResponse.dataUrl,
                    );
                  },
                ),
              );
            } catch (err) {
              completer.completeError(err);
            }
          },
        );
        return completer.future;
      },
    );
  }

  @override
  Future<List<Video>> getGroupVideoList(
      int groupId, int lower, int upper, String sort, String imageRatio) {
    return _fireIsolate<List<Video>>(
      (void _) {
        Completer<List<Video>> completer = Completer();
        NetworkHttpClient httpClient = _getHttpClient();
        String url =
            httpClient.buildUrl(_apiCdnHost, [NetworkPath.getGroupVideoList]);
        Map<String, String> params = <String, String>{};
        params[NetworkQueryKey.groupId] = "$groupId";
        params[NetworkQueryKey.lower] = "$lower";
        params[NetworkQueryKey.upper] = "$upper";
        params[NetworkQueryKey.sort] = sort;
        httpClient.get(url, params).then(
          (body) {
            try {
              List<VideoResponse> videoResponseList =
                  (json.decode(body) as List)
                      .map((data) => VideoResponse.fromJson(data))
                      .toList();
              completer.complete(
                List.generate(
                  videoResponseList.length,
                  (index) {
                    VideoResponse videoResponse = videoResponseList[index];
                    EncodingThumbnailResponse encodingThumbnailResponse =
                        videoResponse.encodingThumbnailList.firstWhere(
                            (element) => element.ratio == imageRatio);
                    return Video(
                      id: videoResponse.id,
                      title: videoResponse.title,
                      thumbnailPath: encodingThumbnailResponse.dataUrl,
                    );
                  },
                ),
              );
            } catch (err) {
              completer.completeError(err);
            }
          },
        );
        return completer.future;
      },
    );
  }

  @override
  Future<RelativeVideoCollection> getRelativeVideoCollection(
      int videoId, int videoSize, String imageRatio) {
    Completer<RelativeVideoCollection> completer = Completer();
    NetworkHttpClient httpClient = _getHttpClient();
    String url = httpClient
        .buildUrl(_apiCdnHost, [NetworkPath.getRelativeVideoCollection]);
    Map<String, String> params = <String, String>{};
    params[NetworkQueryKey.videoId] = "$videoId";
    params[NetworkQueryKey.videoSize] = "$videoSize";
    httpClient.get(url, params).then((body) {
      try {
        RelativeVideoCollectionResponse relativeVideoCollectionResponse =
            RelativeVideoCollectionResponse.fromJson(json.decode(body));
        completer.complete(RelativeVideoCollection(
          groupVideoList: List.generate(
              relativeVideoCollectionResponse.groupVideoList.length,
              (groupVideoIndex) {
            VideoResponse videoResponse =
                relativeVideoCollectionResponse.groupVideoList[groupVideoIndex];
            EncodingThumbnailResponse encodingThumbnailResponse = videoResponse
                .encodingThumbnailList
                .firstWhere((element) => element.ratio == imageRatio);
            return Video(
                id: videoResponse.id,
                title: videoResponse.title,
                thumbnailPath: encodingThumbnailResponse.dataUrl);
          }),
          actorVideoList: List.generate(
              relativeVideoCollectionResponse.actorVideoList.length,
              (actorVideoIndex) {
            VideoResponse videoResponse =
                relativeVideoCollectionResponse.actorVideoList[actorVideoIndex];
            EncodingThumbnailResponse encodingThumbnailResponse = videoResponse
                .encodingThumbnailList
                .firstWhere((element) => element.ratio == imageRatio);
            return Video(
                id: videoResponse.id,
                title: videoResponse.title,
                thumbnailPath: encodingThumbnailResponse.dataUrl);
          }),
        ));
      } catch (err) {
        completer.completeError(err);
      }
    });
    return completer.future;
  }
}
