import 'package:flutter/material.dart';
import 'package:redinfo_integrated_ott_platform_flutter_sdk/delegate/communication_output_delegate.dart';
import '../../../util/system_ui/system_ui.dart' as system_ui;
import '../constant/enum/route.dart';
import '../constant/value/animation.dart';
import '../constant/value/instance_container.dart';
import '../constant/value/route.dart';
import '../delegate/route.dart';
import '../module/advertisement_carousel/presenter.dart';
import '../module/advertisement_image/presenter.dart';
import '../module/classification_menu_bar/presenter.dart';
import '../module/classification_page/presenter.dart';
import '../module/group_search_group_tab/presenter.dart';
import '../module/group_search_page/presenter.dart';
import '../module/group_search_sort_tab/presenter.dart';
import '../module/media_detail/presenter.dart';
import '../module/media_page/presenter.dart';
import '../module/media_player/presenter.dart';
import '../module/media_relative_video_collection/presenter.dart';
import '../module/media_title/presenter.dart';
import '../module/video_collection/presenter.dart';
import '../util/instance_register.dart';

class RouteManager implements RouteDelegate {
  final GlobalKey<NavigatorState> rootNavigatorKey = GlobalKey();
  final GlobalKey<NavigatorState> pageNavigatorKey = GlobalKey();
  final List<String> routeNames = [RoutePath.initial];
  bool _isFullscreen = false;

  bool _allowPath(String path) {
    return [
      RoutePath.classification,
      RoutePath.groupSearch,
      RoutePath.media,
    ].contains(path);
  }

  Widget createModuleWidget(String path, Map<String, dynamic> params) {
    switch (path) {
      case RoutePath.classification:
        return ClassificationPagePresenter().createViewWidget(params);
      case RoutePath.groupSearch:
        return GroupSearchPagePresenter().createViewWidget(params);
      case RoutePath.media:
        return MediaPagePresenter().createViewWidget(params);
    }
    throw UnsupportedError(
        "route path can not found on createModulePresenter -> path: $path");
  }

  Widget getRootNavigator({bool? withPopScope}) {
    if (withPopScope != null && withPopScope) {
      return WillPopScope(
        onWillPop: () async {
          pop();
          return false;
        },
        child: Navigator(
          key: pageNavigatorKey,
          initialRoute: RoutePath.initial,
          onGenerateRoute: _onGenerateRootRoute,
        ),
      );
    }
    return Navigator(
      key: pageNavigatorKey,
      initialRoute: RoutePath.initial,
      onGenerateRoute: _onGenerateRootRoute,
    );
  }

  Route _onGenerateRootRoute(RouteSettings settings) {
    String path = settings.name ?? "";
    if (!_allowPath(path)) {
      path = RoutePath.initial;
    }
    Map<String, dynamic> params = {};
    RouteTransitionDelegate? transitionDelegate;
    if (settings.arguments is _RouteArguments) {
      _RouteArguments arguments = settings.arguments as _RouteArguments;
      params = arguments.params;
      transitionDelegate = arguments.transitionDelegate;
    }
    builder(context) {
      return createModuleWidget(path, params);
    }

    try {
      if (transitionDelegate != null) {
        return _PageRoute.withCustomTransition(
          transitionDelegate: transitionDelegate,
          builder: builder,
          settings: settings,
        );
      } else {
        return _PageRoute(
          builder: builder,
          settings: settings,
        );
      }
    } catch (err) {
      return _PageRoute(
        builder: (context) {
          return const SizedBox.shrink();
        },
        settings: settings,
      );
    }
  }

  @override
  void push(String path, Map<String, dynamic> params,
      {RouteTransitionDelegate? transitionDelegate}) {
    if (pageNavigatorKey.currentState != null) {
      String currentRouteName = routeNames[routeNames.length - 1];
      if (currentRouteName != path) {
        pageNavigatorKey.currentState!.pushNamed(path,
            arguments: _RouteArguments(
                params: params, transitionDelegate: transitionDelegate));
        routeNames.add(path);
        CommunicationOutputDelegate communicationOutputDelegate =
            InstanceRegister.resolve<CommunicationOutputDelegate>(
                InstanceContainer.communicationOutputDelegate);
        communicationOutputDelegate.onRoutePush();
      }
    }
  }

  @override
  void replacement(String path, Map<String, dynamic> params,
      {RouteTransitionDelegate? transitionDelegate}) {
    if (pageNavigatorKey.currentState != null) {
      String currentRouteName = routeNames[routeNames.length - 1];
      if (currentRouteName != path) {
        pageNavigatorKey.currentState!.pushReplacementNamed(path,
            arguments: _RouteArguments(
                params: params, transitionDelegate: transitionDelegate));
        routeNames.removeLast();
        routeNames.add(path);
      }
    }
  }

  @override
  void pop() {
    if (pageNavigatorKey.currentState != null) {
      if (_isFullscreen) {
        pageNavigatorKey.currentState!.maybePop();
        return;
      }
      if (routeNames.length > 1 && pageNavigatorKey.currentState!.canPop()) {
        pageNavigatorKey.currentState!.pop();
        routeNames.removeLast();
        CommunicationOutputDelegate communicationOutputDelegate =
            InstanceRegister.resolve<CommunicationOutputDelegate>(
                InstanceContainer.communicationOutputDelegate);
        communicationOutputDelegate.onRoutePop();
      }
    }
  }

  @override
  Future<void> toggleFullscreen(bool isFullscreen) {
    return system_ui.SystemUi.toggleFullscreen(isFullscreen).then((value) {
      return system_ui.SystemUi.toggleOrientation(isFullscreen).then((value) {
        _isFullscreen = isFullscreen;
        return;
      });
    });
  }

  @override
  Widget getViewModule(
      ModuleViewKey moduleViewKey, Map<String, dynamic> params) {
    switch (moduleViewKey) {
      case ModuleViewKey.advertisementCarousel:
        return AdvertisementCarouselPresenter().createViewWidget(params);
      case ModuleViewKey.advertisementImage:
        return AdvertisementImagePresenter().createViewWidget(params);
      case ModuleViewKey.classificationMenuBar:
        return ClassificationMenuBarPresenter().createViewWidget(params);
      case ModuleViewKey.groupSearchSortTab:
        return GroupSearchSortTabPresenter().createViewWidget(params);
      case ModuleViewKey.groupSearchGroupTab:
        return GroupSearchGroupTabPresenter().createViewWidget(params);
      case ModuleViewKey.videoCollection:
        return VideoCollectionPresenter().createViewWidget(params);
      case ModuleViewKey.mediaPlayer:
        return MediaPlayerPresenter().createViewWidget(params);
      case ModuleViewKey.mediaDetail:
        return MediaDetailPresenter().createViewWidget(params);
      case ModuleViewKey.mediaTitle:
        return MediaTitlePresenter().createViewWidget(params);
      case ModuleViewKey.mediaRelativeVideoCollection:
        return MediaRelativeVideoCollectionPresenter().createViewWidget(params);
    }
  }
}

class _PageRoute extends MaterialPageRoute {
  RouteTransitionDelegate? transitionDelegate;

  @override
  Duration get transitionDuration => AnimationDuration.short;

  _PageRoute({required super.builder, required super.settings});

  _PageRoute.withCustomTransition(
      {this.transitionDelegate,
      required super.builder,
      required super.settings});

  AnimatedWidget getDefaultTransition(Animation<double> animation,
      Animation<double> secondaryAnimation, Widget child) {
    const begin = Offset(1.0, 0.0);
    const end = Offset.zero;
    const curve = Curves.easeInOut;
    var tween = Tween(begin: begin, end: end).chain(CurveTween(curve: curve));
    return SlideTransition(
      position: animation.drive(tween),
      child: child,
    );
  }

  @override
  Widget buildTransitions(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation, Widget child) {
    return transitionDelegate?.customAnimation(
            animation, secondaryAnimation, child) ??
        getDefaultTransition(animation, secondaryAnimation, child);
  }
}

class _RouteArguments {
  final Map<String, dynamic> params;
  final RouteTransitionDelegate? transitionDelegate;

  _RouteArguments({required this.params, this.transitionDelegate});
}
