import 'dart:async';

import 'package:flutter/material.dart';

import '../../bloc/video/bloc.dart';
import '../../bloc/video/delegate.dart';
import '../../bloc/video/event.dart';
import '../../bloc/video/state.dart';
import '../../bloc/video_group/bloc.dart';
import '../../bloc/video_group/delegate.dart';
import '../../bloc/video_group/event.dart';
import '../../bloc/video_group/state.dart';
import '../../constant/enum/route.dart';
import '../../constant/value/image.dart';
import '../../constant/value/instance_container.dart';
import '../../constant/value/length.dart';
import '../../constant/value/module_param.dart';
import '../../constant/value/network.dart';
import '../../constant/value/route.dart';
import '../../constant/value/video.dart';
import '../../delegate/data_service.dart';
import '../../delegate/route.dart';
import '../../structure/entity/video.dart';
import '../../structure/entity/video_group.dart';
import '../../structure/entity/video_sort.dart';
import '../../util/instance_register.dart';
import '../group_search_group_tab/contract.dart';
import '../group_search_sort_tab/contract.dart';
import '../presenter.dart';
import '../video_collection/contract.dart';
import 'contract.dart';
import 'view.dart';

class GroupSearchPagePresenter extends BasePresenter
    implements
        GroupSearchPagePresenterContract,
        VideoGroupBlocDelegate,
        VideoBlocDelegate,
        GroupSearchSortTabModuleDelegate,
        GroupSearchGroupTabModuleDelegate,
        VideoCollectionModuleDelegate {
  static const videoLength = ListLength.large;
  VideoGroupBloc? groupBloc;
  VideoBloc? videoBloc;
  VideoSort? currentSort;
  late VideoGroup currentGroup;
  late int initialGroupId;

  @override
  Widget createView(Map<String, dynamic> params) {
    initialGroupId = params[ModuleParamKey.videoGroupId] as int;
    groupBloc ??= VideoGroupBloc(
        delegate: this, initialState: VideoGroupBlocState.init());
    videoBloc ??=
        VideoBloc(delegate: this, initialState: VideoBlocState.init());
    return GroupSearchPageView(
      presenter: this,
      groupBloc: groupBloc,
      videoBloc: videoBloc,
    );
  }

  @override
  void initialViewData() {
    groupBloc?.add(VideoGroupListFetchEvent(callback: () {
      videoBloc?.add(VideoListFetchEvent());
    }));
  }

  @override
  Widget getGroupSearchSortTabWidget() {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.groupSearchSortTab, {
      ModuleParamKey.moduleDelegate: this,
    });
  }

  @override
  Widget getGroupSearchGroupTabWidget(List<VideoGroup> videoGroupList) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.groupSearchGroupTab, {
      ModuleParamKey.moduleDelegate: this,
      ModuleParamKey.videoGroupId: initialGroupId,
      ModuleParamKey.videoGroupList: videoGroupList,
    });
  }

  @override
  SliverGrid getVideoCollectionWidget(List<Video> videoList) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.videoCollection, {
      ModuleParamKey.moduleDelegate: this,
      ModuleParamKey.crossAxisCount: 3,
      ModuleParamKey.enableScroll: true,
      ModuleParamKey.videoList: videoList,
    }) as SliverGrid;
  }

  @override
  Future<void> onFireLoadingEvent() {
    Completer<void> completer = Completer();
    videoBloc?.add(AppendVideoListEvent(
        length: videoLength,
        callback: () {
          completer.complete();
        }));
    return completer.future;
  }

  @override
  Future<void> onFireRefreshEvent() {
    Completer<void> completer = Completer();
    videoBloc?.add(CleanVideoListEvent(callback: () {
      videoBloc?.add(VideoListFetchEvent(callback: () {
        completer.complete();
      }));
    }));
    return completer.future;
  }

  @override
  Future<List<Video>> fetchVideoList() {
    if (videoBloc == null) {
      return Completer<List<Video>>().future;
    }
    DataServiceDelegate dataServiceDelegate =
        InstanceRegister.resolve<DataServiceDelegate>(
            InstanceContainer.dataServiceDelegate);
    String sort =
        currentSort != null ? currentSort!.identity : NetworkParam.sortLatest;
    if (currentGroup.id == 0) {
      return dataServiceDelegate.getAllVideoList(
        0,
        videoLength - 1,
        sort,
        ImageRatio.ratio_3_4,
      );
    } else {
      return dataServiceDelegate.getGroupVideoList(
        currentGroup.id,
        0,
        videoLength - 1,
        sort,
        ImageRatio.ratio_3_4,
      );
    }
  }

  @override
  Future<List<Video>> appendVideoList(int length) {
    if (videoBloc == null) {
      return Completer<List<Video>>().future;
    }
    DataServiceDelegate dataServiceDelegate =
        InstanceRegister.resolve<DataServiceDelegate>(
            InstanceContainer.dataServiceDelegate);
    String sort =
        currentSort != null ? currentSort!.identity : NetworkParam.sortLatest;
    if (currentGroup.id == 0) {
      return dataServiceDelegate.getAllVideoList(
        videoBloc!.state.videoList.length,
        videoBloc!.state.videoList.length + videoLength - 1,
        sort,
        ImageRatio.ratio_3_4,
      );
    } else {
      return dataServiceDelegate.getGroupVideoList(
        currentGroup.id,
        videoBloc!.state.videoList.length,
        videoBloc!.state.videoList.length + videoLength - 1,
        sort,
        ImageRatio.ratio_3_4,
      );
    }
  }

  @override
  Future<List<VideoGroup>> fetchVideoGroupList() {
    DataServiceDelegate dataServiceDelegate =
        InstanceRegister.resolve<DataServiceDelegate>(
            InstanceContainer.dataServiceDelegate);
    return dataServiceDelegate.getAllVideoGroupList().then((value) {
      List<VideoGroup> videoGroupList = [];
      videoGroupList.add(
          VideoGroup(id: 0, title: VideoGroupTitle.all, collectionCount: 0));
      videoGroupList.addAll(value);
      if (value.isNotEmpty) {
        currentGroup = videoGroupList.firstWhere(
            (videoGroup) => videoGroup.id == initialGroupId,
            orElse: () => videoGroupList[0]);
      }
      return videoGroupList;
    });
  }

  @override
  void onSelectGroup(VideoGroup videoGroup) {
    currentGroup = videoGroup;
    videoBloc?.add(CleanVideoListEvent(callback: () {
      videoBloc?.add(VideoListFetchEvent());
    }));
  }

  @override
  void onSelectSort(VideoSort videoSort) {
    currentSort = videoSort;
    videoBloc?.add(CleanVideoListEvent(callback: () {
      videoBloc?.add(VideoListFetchEvent());
    }));
  }

  @override
  void onFireTapVideoEvent(int id) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    routeDelegate.push(RoutePath.media, {
      ModuleParamKey.videoId: id,
    });
  }
}
