import 'dart:async';
import 'package:flutter/material.dart';
import '../../bloc/advertisement_image/bloc.dart';
import '../../bloc/advertisement_image/delegate.dart';
import '../../bloc/advertisement_image/event.dart';
import '../../bloc/advertisement_image/state.dart';
import '../../bloc/relative_video_collection/bloc.dart';
import '../../bloc/relative_video_collection/delegate.dart';
import '../../bloc/relative_video_collection/event.dart';
import '../../bloc/relative_video_collection/state.dart';
import '../../bloc/video_detail/bloc.dart';
import '../../bloc/video_detail/delegate.dart';
import '../../bloc/video_detail/event.dart';
import '../../bloc/video_detail/state.dart';
import '../../constant/enum/route.dart';
import '../../constant/value/image.dart';
import '../../constant/value/instance_container.dart';
import '../../constant/value/length.dart';
import '../../constant/value/module_param.dart';
import '../../delegate/data_service.dart';
import '../../delegate/route.dart';
import '../../structure/entity/advertisement_image.dart';
import '../../structure/entity/relative_video_collection.dart';
import '../../structure/entity/video_detail.dart';
import '../../util/instance_register.dart';
import '../media_relative_video_collection/contract.dart';
import '../presenter.dart';
import 'contract.dart';
import 'view.dart';

class MediaPagePresenter extends BasePresenter
    implements
        MediaPagePresenterContract,
        AdvertisementImageBlocDelegate,
        VideoDetailBlocDelegate,
        RelativeVideoCollectionBlocDelegate,
        MediaRelativeVideoCollectionModuleDelegate {
  static const relativeVideoListLength = ListLength.large;
  int? _videoId;
  AdvertisementImageBloc? _advertisementImageBloc;
  VideoDetailBloc? _videoDetailBloc;
  RelativeVideoCollectionBloc? _relativeVideoCollectionBloc;
  StreamController<bool>? _playerVisibleController;

  @override
  Widget createView(Map<String, dynamic> params) {
    _videoId ??= params[ModuleParamKey.videoId] as int?;
    _advertisementImageBloc ??= AdvertisementImageBloc(
        delegate: this, initialState: AdvertisementImageBlocState.init());
    _videoDetailBloc ??= VideoDetailBloc(
        delegate: this, initialState: VideoDetailBlocState.init());
    _relativeVideoCollectionBloc ??= RelativeVideoCollectionBloc(
        delegate: this, initialState: RelativeVideoCollectionBlocState.init());
    return MediaPageView(
      presenter: this,
      advertisementImageBloc: _advertisementImageBloc,
      videoDetailBloc: _videoDetailBloc,
      relativeVideoCollectionBloc: _relativeVideoCollectionBloc,
    );
  }

  @override
  void initialViewData() {
    _advertisementImageBloc?.add(AdvertisementImageListMapFetchEvent());
    _relativeVideoCollectionBloc?.add(RelativeVideoCollectionFetchEvent());
    _videoDetailBloc?.add(VideoDetailFetchEvent(callback: () {
      _playerVisibleController?.add(true);
    }));
  }

  @override
  Widget getMediaPlayerWidget(
      VideoDetail videoDetail, AdvertisementImage advertisementImage) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.mediaPlayer, {
      ModuleParamKey.videoId: _videoId,
      ModuleParamKey.videoUrl: videoDetail.sourceUrl,
      ModuleParamKey.title: videoDetail.title,
      ModuleParamKey.videoThumbnailPath: videoDetail.thumbnailPath,
      ModuleParamKey.advertisementImage: advertisementImage,
    });
  }

  @override
  Widget getAdvertisementCarouselWidget(
      List<AdvertisementImage> advertisementImageList) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.advertisementCarousel, {
      ModuleParamKey.advertisementImageList: advertisementImageList,
    });
  }

  @override
  Widget getMediaTitleWidget(VideoDetail videoDetail) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.mediaTitle, {
      ModuleParamKey.videoDetail: videoDetail,
    });
  }

  @override
  Widget getMediaDetailWidget(VideoDetail videoDetail) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate.getViewModule(ModuleViewKey.mediaDetail, {
      ModuleParamKey.moduleDelegate: this,
      ModuleParamKey.videoDetail: videoDetail,
    });
  }

  @override
  Widget getMediaRelativeVideoCollectionWidget(
      RelativeVideoCollection relativeVideoCollection) {
    RouteDelegate routeDelegate = InstanceRegister.resolve<RouteDelegate>(
        InstanceContainer.routeDelegate);
    return routeDelegate
        .getViewModule(ModuleViewKey.mediaRelativeVideoCollection, {
      ModuleParamKey.moduleDelegate: this,
      ModuleParamKey.relativeVideoCollection: relativeVideoCollection,
    });
  }

  @override
  void setPlayerVisibleStreamController(StreamController<bool> controller) {
    _playerVisibleController = controller;
  }

  @override
  Future<void> onFireRefreshEvent() {
    Completer<void> completer = Completer();
    _playerVisibleController?.add(false);
    _videoDetailBloc?.add(CleanVideoDetailEvent(callback: () {
      _videoDetailBloc?.add(VideoDetailFetchEvent());
      completer.complete();
      _playerVisibleController?.add(true);
    }));
    return completer.future;
  }

  @override
  Future<Map<String, List<AdvertisementImage>>>
      fetchAdvertisementImageListMap() {
    DataServiceDelegate dataServiceDelegate =
        InstanceRegister.resolve<DataServiceDelegate>(
            InstanceContainer.dataServiceDelegate);
    return dataServiceDelegate.getAdvertisementImageListMap(
        [ImageRatio.ratio_16_9, ImageRatio.ratio_3_1]);
  }

  @override
  Future<VideoDetail> fetchVideoDetail() {
    if (_videoId == null) {
      return Completer<VideoDetail>().future;
    }
    DataServiceDelegate dataServiceDelegate =
        InstanceRegister.resolve<DataServiceDelegate>(
            InstanceContainer.dataServiceDelegate);
    return dataServiceDelegate.getVideoDetail(
      _videoId!,
      ImageRatio.ratio_4_3,
    );
  }

  @override
  Future<RelativeVideoCollection> fetchRelativeVideoCollection() {
    if (_videoId == null) {
      return Completer<RelativeVideoCollection>().future;
    }
    DataServiceDelegate dataServiceDelegate =
        InstanceRegister.resolve<DataServiceDelegate>(
            InstanceContainer.dataServiceDelegate);
    return dataServiceDelegate.getRelativeVideoCollection(
      _videoId!,
      relativeVideoListLength,
      ImageRatio.ratio_3_4,
    );
  }

  @override
  void onFireTapVideoEvent(int id) {
    _playerVisibleController?.add(false);
    _videoId = id;
    _videoDetailBloc?.add(CleanVideoDetailEvent(callback: () {
      _relativeVideoCollectionBloc
          ?.add(CleanRelativeVideoCollectionEvent(callback: () {
        _relativeVideoCollectionBloc?.add(RelativeVideoCollectionFetchEvent());
        _videoDetailBloc?.add(VideoDetailFetchEvent(callback: () {
          _playerVisibleController?.add(true);
        }));
      }));
    }));
  }
}
