import 'dart:async';
import 'dart:math';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import '../../bloc/advertisement_image/bloc.dart';
import '../../bloc/advertisement_image/state.dart';
import '../../bloc/relative_video_collection/bloc.dart';
import '../../bloc/relative_video_collection/state.dart';
import '../../bloc/video_detail/bloc.dart';
import '../../bloc/video_detail/state.dart';
import '../../component/view/refresh_wrapper.dart';
import '../../constant/value/color.dart';
import '../../constant/value/image.dart';
import '../../structure/entity/advertisement_image.dart';
import 'contract.dart';

class MediaPageView extends StatefulWidget {
  final MediaPagePresenterContract presenter;
  final AdvertisementImageBloc? advertisementImageBloc;
  final VideoDetailBloc? videoDetailBloc;
  final RelativeVideoCollectionBloc? relativeVideoCollectionBloc;

  const MediaPageView({
    super.key,
    required this.presenter,
    required this.advertisementImageBloc,
    this.videoDetailBloc,
    this.relativeVideoCollectionBloc,
  });

  @override
  State<MediaPageView> createState() => _MediaPageViewState();
}

class _MediaPageViewState extends State<MediaPageView> {
  late final StreamController<bool> _playerVisibleStreamController;

  @override
  void initState() {
    super.initState();
    _playerVisibleStreamController = StreamController<bool>();
    widget.presenter
        .setPlayerVisibleStreamController(_playerVisibleStreamController);
  }

  @override
  void dispose() {
    _playerVisibleStreamController.close();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: CustomColors.black,
      body: Column(
        children: <Widget>[
          _getMediaPlayerWidget(),
          Expanded(
            child: Stack(
              children: [
                RefreshWrapper(
                  enableRefresh: false,
                  enableLoading: false,
                  onRefresh: widget.presenter.onFireRefreshEvent,
                  child: ListView.builder(
                    itemCount: 5,
                    itemBuilder: (BuildContext context, int index) {
                      switch (index) {
                        case 0:
                          return _getMediaTitleWidget();
                        case 1:
                          return _getMediaDetailWidget();
                        case 2:
                          return _getMediaCarouselWidget();
                        case 3:
                          return _getMediaDividerWidget();
                        case 4:
                          return _getMediaRelativeVideoCollection();
                      }
                      return const SizedBox.shrink();
                    },
                  ),
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }

  Widget _getMediaPlayerWidget() {
    return AspectRatio(
      aspectRatio: 16 / 9,
      child: StreamBuilder<bool>(
        stream: _playerVisibleStreamController.stream,
        builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
          if (!snapshot.hasData || !snapshot.data!) {
            return const SizedBox.shrink();
          }
          return BlocBuilder<AdvertisementImageBloc,
              AdvertisementImageBlocState>(
            bloc: widget.advertisementImageBloc,
            builder: (context, advertisementImageBlocState) {
              if (!advertisementImageBlocState.advertisementImageListMap
                  .containsKey(ImageRatio.ratio_16_9)) {
                return const SizedBox.shrink();
              }
              return BlocBuilder<VideoDetailBloc, VideoDetailBlocState>(
                bloc: widget.videoDetailBloc,
                builder: (context, videoDetailBlocState) {
                  if (videoDetailBlocState.videoDetail == null) {
                    return const SizedBox.shrink();
                  }
                  List<AdvertisementImage> advertisementImageList =
                      advertisementImageBlocState
                          .advertisementImageListMap[ImageRatio.ratio_16_9]!;
                  AdvertisementImage advertisementImage =
                      advertisementImageList[
                          Random().nextInt(advertisementImageList.length)];
                  return widget.presenter.getMediaPlayerWidget(
                    videoDetailBlocState.videoDetail!,
                    advertisementImage,
                  );
                },
              );
            },
          );
        },
      ),
    );
  }

  Widget _getMediaCarouselWidget() {
    return BlocBuilder<AdvertisementImageBloc, AdvertisementImageBlocState>(
      bloc: widget.advertisementImageBloc,
      builder: (context, state) {
        if (!state.advertisementImageListMap
            .containsKey(ImageRatio.ratio_3_1)) {
          return const SizedBox.shrink();
        }
        List<AdvertisementImage> list =
            state.advertisementImageListMap[ImageRatio.ratio_3_1]!.toList();
        return AspectRatio(
          aspectRatio: ImageAspectRatio.ratio_3_1,
          child:
              widget.presenter.getAdvertisementCarouselWidget(list..shuffle()),
        );
      },
    );
  }

  Widget _getMediaTitleWidget() {
    return SizedBox(
      width: double.infinity,
      child: BlocBuilder<VideoDetailBloc, VideoDetailBlocState>(
        bloc: widget.videoDetailBloc,
        builder: (context, state) {
          if (state.videoDetail == null) {
            return const SizedBox.shrink();
          }
          return widget.presenter.getMediaTitleWidget(state.videoDetail!);
        },
      ),
    );
  }

  Widget _getMediaDetailWidget() {
    return BlocBuilder<VideoDetailBloc, VideoDetailBlocState>(
      bloc: widget.videoDetailBloc,
      builder: (context, state) {
        if (state.videoDetail == null) {
          return const SizedBox.shrink();
        }
        return Padding(
          padding: const EdgeInsets.symmetric(vertical: 4),
          child: widget.presenter.getMediaDetailWidget(state.videoDetail!),
        );
      },
    );
  }

  Widget _getMediaDividerWidget() {
    return const Padding(
      padding: EdgeInsets.all(8),
      child: Divider(
        color: CustomColors.smokeGray,
        height: 1,
        thickness: 1,
      ),
    );
  }

  Widget _getMediaRelativeVideoCollection() {
    return BlocBuilder<RelativeVideoCollectionBloc,
        RelativeVideoCollectionBlocState>(
      bloc: widget.relativeVideoCollectionBloc,
      builder: (context, state) {
        if (state.relativeVideoCollection == null) {
          return const SizedBox.shrink();
        }
        return widget.presenter.getMediaRelativeVideoCollectionWidget(
            state.relativeVideoCollection!);
      },
    );
  }
}
