import 'package:flutter/material.dart';
import 'package:redinfo_integrated_ott_platform_flutter_sdk/structure/entity/resource_state.dart';
import '../../component/view/base64_file_image_provider.dart';
import '../../component/view/countdown_button.dart';
import '../../component/view/player/player.dart' as player;
import '../../component/view/ripple_wrapper.dart';
import '../../constant/value/color.dart';
import '../../constant/value/localization.dart';
import '../../constant/value/size.dart';
import '../../structure/entity/advertisement_image.dart';
import '../../util/intent.dart';
import '../../util/localization.dart';
import '../../util/toast.dart';
import 'contract.dart';

class MediaPlayerView extends StatefulWidget {
  final MediaPlayerPresenterContract presenter;
  final String url;
  final AdvertisementImage advertisementImage;
  final String thumbnailPath;

  const MediaPlayerView({
    super.key,
    required this.presenter,
    required this.url,
    required this.advertisementImage,
    required this.thumbnailPath,
  });

  @override
  State<MediaPlayerView> createState() => _MediaPlayerViewState();
}

class _MediaPlayerViewState extends State<MediaPlayerView> {
  static const _countdownSec = 5;
  late bool _shouldShowThumbnail;
  late bool _shouldShowAdvertisement;

  @override
  void initState() {
    super.initState();
    setState(() {
      _shouldShowThumbnail = true;
      _shouldShowAdvertisement = true;
    });
  }

  @override
  Widget build(BuildContext context) {
    if (_shouldShowThumbnail) {
      return _getThumbnailWidget();
    }
    return _shouldShowAdvertisement
        ? _getAdvertisementWidget()
        : player.PlayerView(
            videoUrl: widget.url,
            aspectRatio: 16 / 9,
            pauseImagePath: widget.advertisementImage.imagePath,
            onPauseImageTap: _onPauseImageTap,
          );
  }

  Widget _getThumbnailWidget() {
    return FutureBuilder<ResourceState>(
      future: widget.presenter.getMediaResourceState(),
      builder: (BuildContext resourceStateContext,
          AsyncSnapshot<ResourceState> resourceStateSnapshot) {
        return FutureBuilder<int>(
          future: widget.presenter.getMediaTotalResourceSize(),
          builder: (BuildContext totalResourceSizeContext,
              AsyncSnapshot<int> totalResourceSizeSnapshot) {
            return FutureBuilder<int>(
                future: widget.presenter.getMediaUsedResourceSize(),
                builder: (BuildContext usedResourceSizeContext,
                    AsyncSnapshot<int> usedResourceSizeSnapshot) {
                  if (!resourceStateSnapshot.hasData ||
                      !totalResourceSizeSnapshot.hasData ||
                      !usedResourceSizeSnapshot.hasData) {
                    return const SizedBox.shrink();
                  }
                  return LayoutBuilder(
                    builder:
                        (BuildContext context, BoxConstraints constraints) {
                      return Stack(
                        children: [
                          Image(
                            image: Base64FileImageProvider(
                              widget.thumbnailPath,
                            ),
                            fit: BoxFit.cover,
                            width: constraints.maxWidth,
                            height: constraints.maxHeight,
                          ),
                          Positioned(
                            width: 60,
                            height: 60,
                            top: constraints.maxHeight / 2 - 30,
                            left: constraints.maxWidth / 2 - 30,
                            child: _getThumbnailCenterButtonWidget(
                              resourceStateSnapshot.data!.consumeState,
                            ),
                          ),
                          Positioned(
                            width: constraints.maxWidth,
                            top: constraints.maxHeight / 2 + 30,
                            child: _getThumbnailInformationWidget(
                                usedResourceSizeSnapshot.data!,
                                totalResourceSizeSnapshot.data!),
                          ),
                        ],
                      );
                    },
                  );
                });
          },
        );
      },
    );
  }

  Widget _getThumbnailCenterButtonWidget(bool consumeState) {
    return Card(
      color: CustomColors.transparent,
      elevation: 0,
      child: RippleWrapper(
        onTap: consumeState ? _onCloseThumbnail : _onFireConsumeMedia,
        borderRadius: BorderRadius.circular(30),
        splashColor: CustomColors.gray.withOpacity(0.4),
        decoration: BoxDecoration(
          shape: BoxShape.circle,
          color: CustomColors.midnightBlack.withOpacity(0.4),
          boxShadow: null,
        ),
        child: const Icon(
          Icons.play_arrow,
          color: CustomColors.white,
          size: 30,
        ),
      ),
    );
  }

  Widget _getThumbnailInformationWidget(int usedSize, int totalSize) {
    return Container(
      color: CustomColors.midnightBlack.withOpacity(0.6),
      child: Padding(
        padding: const EdgeInsets.all(5),
        child: Text(
          "${Localization.getString(context, LocalizationStrings.allowVideoCountToday)}：$usedSize / $totalSize",
          textAlign: TextAlign.center,
          style: const TextStyle(
            fontSize: FontSize.medium,
            color: CustomColors.white,
          ),
        ),
      ),
    );
  }

  Widget _getAdvertisementWidget() {
    return Stack(
      children: [
        LayoutBuilder(
          builder: (BuildContext context, BoxConstraints constraints) {
            return GestureDetector(
              onTap: _onPauseImageTap,
              child: Image(
                image: Base64FileImageProvider(
                  widget.advertisementImage.imagePath,
                ),
                fit: BoxFit.cover,
                width: constraints.maxWidth,
                height: constraints.maxHeight,
              ),
            );
          },
        ),
        Positioned(
          width: 50,
          height: 50,
          top: 5,
          right: 5,
          child: CountdownButtonView(
            countdownSec: _countdownSec,
            onClose: _onStartPlay,
          ),
        )
      ],
    );
  }

  void _onFireConsumeMedia() {
    widget.presenter.onFireConsumeMedia().then((success) {
      if (success) {
        _onCloseThumbnail();
      } else {
        ToastDistributor.showToast("not available");
      }
    });
  }

  void _onCloseThumbnail() {
    setState(() {
      _shouldShowThumbnail = false;
    });
  }

  void _onStartPlay() {
    setState(() {
      _shouldShowAdvertisement = false;
    });
  }

  void _onPauseImageTap() {
    IntentDistributor.fireBrowserIntent(widget.advertisementImage.link);
  }
}
